@@ -30,6 +30,7 @@
 #include <QMap>
 #include <QFont>
 #include <QFile>
+#include <QPushButton>
 
 #include <QDebug>
 
@@ -51,16 +52,21 @@ public:
 signals:
 
 public slots:
+    void unfollowContact();
 
 private:
     QHBoxLayout *mainLayout;
-    QVBoxLayout *rightLayout;
+    QVBoxLayout *centerLayout;
 
     QLabel *avatarLabel;
     QLabel *nameLabel;
 
     QLabel *tmpLabel;
 
+    QPushButton *followButton;
+
+    PumpController *pController;
+    QString contactID;
 };
 
 #endif // CONTACTCARD_H
