@@ -29,15 +29,29 @@ ContactList::ContactList(PumpController *pumpController,
     mainLayout->setAlignment(Qt::AlignTop);
 
 
+    topLayout = new QHBoxLayout();
+    addressLineEdit = new QLineEdit();
+    addressLineEdit->setPlaceholderText(tr("Enter a Webfinger address, like username@pumpserver.org"));
+
+    followButton = new QPushButton(QIcon::fromTheme("list-add"), tr("&Follow"));
+    connect(followButton, SIGNAL(clicked()),
+            this, SLOT(followContact()));
+    topLayout->addWidget(addressLineEdit);
+    topLayout->addWidget(followButton);
+
+    mainLayout->addLayout(topLayout);
+
+    contactsLayout = new QVBoxLayout();
+
     QMap<QString,QString> demoContactData;
     demoContactData.insert("name", "Demo Contact");
     demoContactData.insert("id",   "democontact@somepumpserver.org");
     demoContactData.insert("avatar", "");
-    demoContactData.insert("sharing", "true");
-    demoContactData.insert("receiving", "false");
-    mainLayout->addWidget(new ContactCard(this->pController, demoContactData, this));
-
+    demoContactData.insert("follower", "true");
+    demoContactData.insert("following", "true");
+    contactsLayout->addWidget(new ContactCard(this->pController, demoContactData, this));
 
+    mainLayout->addLayout(contactsLayout);
     this->setLayout(mainLayout);
 
     qDebug() << "Contact list created";
@@ -51,9 +65,24 @@ ContactList::~ContactList()
 
 
 
+void ContactList::clearContactListContents()
+{
+    QLayoutItem *child;
+
+    while ((child = contactsLayout->takeAt(0)) != 0)
+    {
+        child->widget()->deleteLater(); // Delete the widget (contactCard) itself
+        delete child;
+    }
+}
 
 
+
+
+
+/*****************************************************************************/
 /*********************************** SLOTS ***********************************/
+/*****************************************************************************/
 
 
 
@@ -61,6 +90,7 @@ ContactList::~ContactList()
 void ContactList::setContactListContents(QVariantList contactList)
 {
     qDebug() << "Setting contact list contents";
+    this->clearContactListContents();
 
 
     QMap<QString,QString> contactData;
@@ -70,11 +100,40 @@ void ContactList::setContactListContents(QVariantList contactList)
         contactData.insert("name", contact.toMap().value("displayName").toString());
         contactData.insert("id", contact.toMap().value("id").toString().remove(0,5)); // remove "acct:"
         contactData.insert("avatar", contact.toMap().value("image").toMap().value("url").toString());
-        contactData.insert("sharing", "false"); // tmp, until we have followers/following separation
-        contactData.insert("receiving", "true");
+        contactData.insert("follower", "false"); // tmp, until we have followers/following separation
+        contactData.insert("following", "true");
 
-        this->mainLayout->addWidget(new ContactCard(this->pController,
-                                                    contactData,
-                                                    this));
+        this->contactsLayout->addWidget(new ContactCard(this->pController,
+                                                        contactData,
+                                                        this));
     }
 }
+
+
+
+
+/*
+ * Add the address entered by the user to the /following list.
+ *
+ * This should probably support adding webfinger addresses
+ * in the form https://host/username too.
+ *
+ */
+void ContactList::followContact()
+{
+    QString address = this->addressLineEdit->text();
+
+    qDebug() << "ContactList::followContact()" << address;
+
+    if (address.contains("@")) // FIXME, check correctly
+    {
+        this->pController->followContact(address); // this will trigger a reload of the contact list
+
+        this->addressLineEdit->clear();
+    }
+    else
+    {
+        qDebug() << "Invalid webfinger address!";
+    }
+
+}
