@@ -23,10 +23,13 @@
 
 #include <QWidget>
 #include <QLabel>
+#include <QHBoxLayout>
 #include <QVBoxLayout>
 #include <QVariantList>
 #include <QMap>
 #include <QString>
+#include <QPushButton>
+#include <QLineEdit>
 
 #include <QDebug>
 
@@ -42,14 +45,24 @@ public:
     ContactList(PumpController *pumpController, QWidget *parent = 0);
     ~ContactList();
 
+    void clearContactListContents();
+
 signals:
 
 public slots:
     void setContactListContents(QVariantList contactList);
 
+    void followContact();
+
 private:
     QVBoxLayout *mainLayout;
 
+    QHBoxLayout *topLayout;
+    QLineEdit *addressLineEdit;
+
+    QVBoxLayout *contactsLayout;
+    QPushButton *followButton;
+
     PumpController *pController;
 };
 
