@@ -25,19 +25,63 @@
 #include "mainwindow.h"
 
 
+
+
+
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
+void customMessageHandlerQt4(QtMsgType type, const char *msg)
+{
+    // do nothing
+    Q_UNUSED(type)
+    Q_UNUSED(msg)
+
+    return;
+}
+#else
+void customMessageHandlerQt5(QtMsgType type, const QMessageLogContext &context, const QString &msg)
+{
+    Q_UNUSED(type)
+    Q_UNUSED(context)
+    Q_UNUSED(msg)
+
+    // Do nothing
+
+    return;
+}
+#endif
+
+
+
 int main(int argc, char *argv[])
 {
     QApplication dianaraApp(argc, argv);
     dianaraApp.setApplicationName("Dianara");
-    dianaraApp.setApplicationVersion("0.4");
+    dianaraApp.setApplicationVersion("0.5");
     dianaraApp.setOrganizationName("JanCoding");
     dianaraApp.setOrganizationDomain("jancoding.wordpress.com");
 
-    std::cout << QString("%1 v%2 - JanKusanagi 2012\n")
+    std::cout << QString("%1 v%2 - JanKusanagi 2012-2013\n")
                      .arg(dianaraApp.applicationName())
                      .arg(dianaraApp.applicationVersion()).toStdString();
     std::cout.flush();
 
+
+    // Register custom message handler, to hide debug messages unless specified
+    if (qApp->arguments().contains("--debug", Qt::CaseInsensitive))
+    {
+        qDebug() << "Debug messages enabled";
+    }
+    else
+    {
+        qDebug() << "To see debug messages while running, use --debug";
+   #if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
+        qInstallMsgHandler(customMessageHandlerQt4);
+   #else
+        qInstallMessageHandler(customMessageHandlerQt5);
+   #endif
+    }
+
+
     // Load translation files
     QTranslator translator;
     QString languageFile;
