@@ -27,6 +27,8 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
     this->setWindowIcon(QIcon(":/icon/64x64/dianara.png"));
     this->setMinimumSize(400, 400);
 
+    QSettings settings;
+
     firstRun = true;
     prepareDataDirectory();
 
@@ -35,15 +37,15 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
 
 
     // Network control
-    pumpController = new PumpController(/*userID, userAuth?*/);
+    pumpController = new PumpController();
 
 
-    // GUI
-    centralWidget = new QWidget();
-    mainLayout = new QHBoxLayout();
+    ////// GUI
+    mainSplitter = new QSplitter(Qt::Horizontal, this);
 
 
     // Left side
+    leftSideWidget = new QWidget();
     leftLayout = new QVBoxLayout();
 
     avatarIconLabel = new QLabel();
@@ -59,6 +61,19 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
     leftLayout->addSpacing(16);
 
     leftPanel = new QToolBox();
+
+    QLabel *tmpMeanwhileLabel = new QLabel("\n"
+                                           "- Somebody posted something.\n"
+                                           "- You followed a person.\n"
+                                           "- John Doe did something.\n"
+                                           "- etc.");
+    tmpMeanwhileLabel->setAlignment(Qt::AlignTop | Qt::AlignLeft);
+
+    leftPanel->addItem(tmpMeanwhileLabel,
+                       QIcon::fromTheme("clock"),
+                       tr("Meanwhile..."));
+
+
     aspectListLabel = new QLabel("- <a href=\"list1\">List 1</a><br /><br />"
                                  "- <a href=\"list2\">List 2</a><br /><br />"
                                  "- <a href=\"list3\">List 3</a>");
@@ -78,7 +93,7 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
             pumpController, SLOT(setCurrentTimeline(QString)));
     leftPanel->addItem(tmpTagsLabel,
                        QIcon::fromTheme("mail-tagged"),
-                       "#&Tags");
+                       "#T&ags");
     QLabel *tmpMoreLabel = new QLabel("- Activity\n\n"
                                       "- @Mentions\n\n"
                                       "");
@@ -89,11 +104,15 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
 
     leftLayout->addWidget(leftPanel);
 
+    this->leftSideWidget->setLayout(leftLayout);
+
+
+
     // Right side
+    rightSideWidget = new QWidget();
     rightLayout = new QVBoxLayout();
 
-    QSettings tmpSettings; // FIXME
-    this->publisherType = tmpSettings.value("publisherType", 0).toInt(); // FIXME
+    this->publisherType = settings.value("publisherType", 0).toInt(); // FIXME
     publisher = new Publisher(publisherType);
 
 
@@ -101,6 +120,11 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
     timeLineScrollArea = new QScrollArea();
     timeLineScrollArea->setWidget(timeLine);  // Make timeline scrollable
     timeLineScrollArea->setWidgetResizable(true); // Seems to work in opposite way
+    connect(timeLine, SIGNAL(scrollToTop()),
+            this, SLOT(scrollTimelineToTop()));
+
+    connect(timeLine, SIGNAL(timelineRendered(int)),
+            this, SLOT(notifyTimelineUpdate(int)));
 
 
     messageList = new MessageList(this);
@@ -116,8 +140,8 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
 
     tabWidget = new QTabWidget();
     tabWidget->addTab(timeLineScrollArea,
-                      QIcon::fromTheme("kde"),
-                      tr("Time&line"));
+                      QIcon::fromTheme("view-list-details"),
+                      tr("&Timeline"));
     tabWidget->addTab(messageListScrollArea,
                       QIcon::fromTheme("mail-message"),
                       tr("&Messages"));
@@ -128,13 +152,15 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
     rightLayout->addWidget(publisher, 1); // stretch 1/10
     rightLayout->addWidget(tabWidget, 9); // stretch 9/10
 
+    this->rightSideWidget->setLayout(rightLayout);
 
 
-    mainLayout->addLayout(leftLayout, 1); // stretch=1/10
-    mainLayout->addLayout(rightLayout, 9); // stretch=9/10, take +space than left panel
-    centralWidget->setLayout(mainLayout);
+    mainSplitter->addWidget(leftSideWidget);
+    mainSplitter->addWidget(rightSideWidget);
+    mainSplitter->setHandleWidth(4);
+    mainSplitter->setChildrenCollapsible(false);
+    this->setCentralWidget(mainSplitter);
 
-    this->setCentralWidget(centralWidget);
 
 
     ////////////////// Load configuration from disk
@@ -160,12 +186,13 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
             this, SLOT(updateUserID(QString)));
 
     configDialog = new ConfigDialog(this->updateInterval,
+                                    this->postsPerPage,
                                     this->tabsPosition,
                                     this->tabsMovable,
                                     this->publisherType,
                                     this->showNotifications);
-    connect(configDialog, SIGNAL(configurationChanged(int,int,bool,int,int)),
-            this, SLOT(updateSettings(int,int,bool,int,int)));
+    connect(configDialog, SIGNAL(configurationChanged(int,int,int,bool,int,int)),
+            this, SLOT(updateSettings(int,int,int,bool,int,int)));
 
 
 
@@ -179,8 +206,8 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
             this, SLOT(storeImage(QByteArray,QUrl)));
     connect(pumpController, SIGNAL(aspectListReceived(QVariantList)),
             this, SLOT(updateAspectList(QVariantList)));
-    connect(pumpController, SIGNAL(timeLineReceived(QVariantList)),
-            timeLine, SLOT(setTimeLineContents(QVariantList)));
+    connect(pumpController, SIGNAL(timeLineReceived(QVariantList,int,QString,QString)),
+            timeLine, SLOT(setTimeLineContents(QVariantList,int,QString,QString)));
     connect(pumpController, SIGNAL(contactListReceived(QVariantList)),
             contactList, SLOT(setContactListContents(QVariantList)));
     connect(pumpController, SIGNAL(showNotification(QString)),
@@ -212,9 +239,13 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
 
 
     // tmp statusBar stuff
-    this->statusBar()->showMessage("Initializing... (this is work in progress)");
+    this->statusBar()->showMessage(tr("Initializing..."));
 
 
+    // Now set the "view side panel" checkable menu to its saved state
+    // That will also trigger the action to hide/show it
+    viewSidePanel->setChecked(settings.value("viewSidePanel", true).toBool());
+
 
 
     qDebug() << "MainWindow created";
@@ -347,6 +378,21 @@ void MainWindow::createMenus()
     this->menuBar()->addMenu(sessionMenu);
 
 
+    viewMenu = new QMenu(tr("&View"));
+
+    viewSidePanel = new QAction(QIcon::fromTheme("view-sidetree"),
+                                tr("View Side &Panel"), this);
+    connect(viewSidePanel, SIGNAL(toggled(bool)),
+            this, SLOT(toggleSidePanel(bool)));
+    viewSidePanel->setCheckable(true);
+    viewSidePanel->setChecked(true);
+    viewSidePanel->setShortcut(QKeySequence("F9"));
+
+    viewMenu->addAction(viewSidePanel);
+
+    this->menuBar()->addMenu(viewMenu);
+
+
 
     settingsMenu = new QMenu(tr("S&ettings"));
 
@@ -452,6 +498,11 @@ void MainWindow::loadSettings()
                                                        tr("[ Configure your account ]")).toString());
 
     this->resize(settings.value("windowSize", QSize(640, 450)).toSize());
+    this->mainSplitter->restoreState(settings.value("mainSplitterState",
+                                                    mainSplitter->saveState()).toByteArray());
+    // Set childrenCollapsible to false AFTER loading state, to make sure state does not mess with it
+    mainSplitter->setChildrenCollapsible(false);
+
 
     this->updateInterval = settings.value("updateInterval", 5).toInt();
 
@@ -478,6 +529,9 @@ void MainWindow::loadSettings()
         userAuth = settings.value("userAuth", "").toByteArray();
     }
 
+    this->postsPerPage = settings.value("postsPerPage", 20).toInt();
+    this->pumpController->setPostsPerPage(this->postsPerPage);
+
 
     qDebug() << "Settings loaded";
 }
@@ -494,6 +548,9 @@ void MainWindow::saveSettings()
 
     settings.setValue("windowSize", this->size());
     settings.setValue("windowPosition", this->pos());
+    settings.setValue("mainSplitterState", this->mainSplitter->saveState());
+
+    settings.setValue("viewSidePanel", this->viewSidePanel->isChecked());
 
     settings.setValue("updateInterval", this->updateInterval);
 
@@ -504,6 +561,8 @@ void MainWindow::saveSettings()
 
     settings.setValue("showNotifications", this->showNotifications);
 
+    settings.setValue("postsPerPage", this->postsPerPage);
+
     qDebug() << "Settings saved";
 }
 
@@ -541,13 +600,16 @@ void MainWindow::updateUserID(QString newUserID)
  * Update settings changed from ConfigDialog()
  *
  */
-void MainWindow::updateSettings(int newUpdateInterval, int newTabsPosition,
-                                bool newTabsMovable, int newPublisherType,
-                                int newShowNotifications)
+void MainWindow::updateSettings(int newUpdateInterval, int newPostsPerPage,
+                                int newTabsPosition, bool newTabsMovable,
+                                int newPublisherType, int newShowNotifications)
 {
     this->updateInterval = newUpdateInterval;
     this->updateTimer->setInterval(updateInterval * 1000 * 60);
 
+    this->postsPerPage = newPostsPerPage;
+    this->pumpController->setPostsPerPage(this->postsPerPage);
+
     this->tabsPosition = newTabsPosition;
     this->tabWidget->setTabPosition((QTabWidget::TabPosition)tabsPosition);
 
@@ -673,19 +735,48 @@ void MainWindow::updateTimelines()
     pumpController->getTimeline();
     // received timeline data will come in a SIGNAL()
 
-
+/*
     if (showNotifications != 2) // If notifications are enabled (FD.org or Qt's)
     {
         fdNotifier->showMessage("Updating <u>timelines</u>.\n"
                                 "Temporary <b>dummy</b> notification!\n"
                                 "<a href=\"http://jancoding.wordpress.com/dianara\">Go to Dianara's website</a>");
     }
+*/
 
     qDebug() << "Updated timelines by menu or after:" << this->updateInterval << "min";
 }
 
 
 
+void MainWindow::scrollTimelineToTop()
+{
+    this->timeLineScrollArea->verticalScrollBar()->setValue(0);
+    this->timeLineScrollArea->horizontalScrollBar()->setValue(0);
+}
+
+
+
+void MainWindow::notifyTimelineUpdate(int newPostCount)
+{
+    if (newPostCount > 0)
+    {
+        this->setStatusBarMessage(tr("Timeline updated. There are %1 new posts.").arg(newPostCount));
+
+        if (this->showNotifications != 2) // If some type of notifications are enabled
+        {
+            this->fdNotifier->showMessage(tr("Timeline updated at %1.\nThere are %2 new posts.")
+                                          .arg(QTime::currentTime().toString()).arg(newPostCount));
+        }
+    }
+    else
+    {
+        this->setStatusBarMessage(tr("Timeline updated. No new posts."));
+    }
+}
+
+
+
 
 /*
  * Store avatars on disk
@@ -731,9 +822,6 @@ void MainWindow::storeImage(QByteArray imageData, QUrl imageURL)
     QString imageString = imageURL.toString();
 
     QString fileName = imageString.toUtf8().toBase64();
-    QString fileExtension = imageString.remove(QRegExp(".*\\."));
-    fileName.append("." + fileExtension);
-
     qDebug() << "Saving image to disk: " << fileName;
 
     QFile imageFile(dataDirectory + "/images/" + fileName);
@@ -747,13 +835,32 @@ void MainWindow::storeImage(QByteArray imageData, QUrl imageURL)
 
 
 /*
- * Update status bar message, from pumpController's signals
+ * Update status bar message, from pumpController's (and others) signals
  *
  */
 void MainWindow::setStatusBarMessage(QString message)
 {
-    //this->setStatusTip(message);
-    this->statusBar()->showMessage(message, 0);
+    this->statusBar()->showMessage("[" + QTime::currentTime().toString() + "] " + message, 0);
+}
+
+
+
+/*
+ * Hide or show the side panel
+ *
+ */
+void MainWindow::toggleSidePanel(bool shown)
+{
+    // Unless the publisher (really, the composer) has focus already,
+    // give focus to timeline before, to avoid giving focus to the publisher
+    if (!this->publisher->hasFocus())
+    {
+        // FIXME: check if publisher's composer has focus!
+        this->timeLine->setFocus();
+    }
+
+    qDebug() << "Showing side panel:" << shown;
+    this->leftSideWidget->setShown(shown);
 }
 
 
@@ -780,7 +887,7 @@ void MainWindow::visitWebSite()
 void MainWindow::aboutDianara()
 {
     QMessageBox::about(this, tr("About Dianara"),
-                       "<b>Dianara v0.4</b><br>"
+                       "<b>Dianara v0.5</b><br>"
                        "Copyright 2012-2013  JanKusanagi<br>"
                        "<a href=\"http://dianara.nongnu.org\">"
                        "http://dianara.nongnu.org</a><br>"
@@ -789,7 +896,8 @@ void MainWindow::aboutDianara()
                        + tr("Dianara is a pump.io social networking client.<br><br>"
                             "It is work in progress. Right now only the "
                             "interface and some basic capabilities: posting text, "
-                            "showing last 20 posts and contact listing."
+                            "showing timeline, linking/commenting/sharing/deleting posts, "
+                            "and contact listing."
                             "<br><br>"));
 
     if (this->isHidden()) // FIXME: ugly workaround to avoid closing the program
