@@ -40,11 +40,14 @@
 #include <QDir>
 #include <QTimer>
 #include <QScrollArea>
+#include <QScrollBar>
 #include <QVariantList>
 #include <QDesktopServices>
 #if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
 #include <QStandardPaths>
 #endif
+#include <QTime>
+#include <QSplitter>
 
 #include <QDebug>
 
@@ -78,6 +81,7 @@ public:
 public slots:
     void updateUserID(QString newUserID);
     void updateSettings(int newUpdateInterval,
+                        int newPostsPerPage,
                         int newTabsPosition,
                         bool newTabsMovable,
                         int newPublisherType,
@@ -90,12 +94,17 @@ public slots:
     void updateAspectList(QVariantList aspectList);
 
     void updateTimelines();
+    void scrollTimelineToTop();
+    void notifyTimelineUpdate(int newPostCount);
 
     void storeAvatar(QByteArray avatarData, QUrl avatarURL);
     void storeImage(QByteArray imageData, QUrl imageURL);
 
     void setStatusBarMessage(QString message);
 
+
+    void toggleSidePanel(bool shown);
+
     void visitWebSite();
     void aboutDianara();
     void quitProgram();
@@ -107,6 +116,7 @@ protected:
 
 private:
     QMenu *sessionMenu;
+    QMenu *viewMenu;
     QMenu *settingsMenu;
     QMenu *helpMenu;
 
@@ -115,18 +125,20 @@ private:
     QAction *sessionUpdateTimelines;
     QAction *sessionStartConversation;
     QAction *sessionQuit;
+    QAction *viewSidePanel;
     QAction *settingsAccount;
     QAction *settingsConfigure;
     QAction *helpVisitWebsite;
     QAction *helpAbout;
 
 
-    QWidget *centralWidget;
-    QHBoxLayout *mainLayout;
+    QSplitter *mainSplitter;
 
+    QWidget *leftSideWidget;
     QVBoxLayout *leftLayout;
     QToolBox *leftPanel;
 
+    QWidget *rightSideWidget;
     QVBoxLayout *rightLayout;
     QTabWidget *tabWidget;
     int tabsPosition;
@@ -147,6 +159,7 @@ private:
     FDNotifications *fdNotifier;
 
     TimeLine *timeLine;
+    int postsPerPage;
     QScrollArea *timeLineScrollArea;
     MessageList *messageList;
     QScrollArea *messageListScrollArea;
