@@ -34,6 +34,8 @@
 #include <QMap>
 #include <QFile>
 #include <QResizeEvent>
+#include <QVariantList>
+#include <QMessageBox>
 
 #include <QDebug>
 
@@ -42,6 +44,7 @@
 #include "timestamp.h"
 #include "mischelpers.h"
 #include "pumpcontroller.h"
+#include "commenter.h"
 
 
 class Post : public QFrame
@@ -51,6 +54,7 @@ class Post : public QFrame
 public:
     Post(PumpController *pumpController,
          QMap<QString,QString> postData,
+         QVariantList commentsVariantList,
          QWidget *parent = 0);
     ~Post();
 
@@ -60,8 +64,10 @@ public slots:
     void showNsfwPost();
 
     void likePost();
-    void commentPost();
+    void commentOnPost();
+    void sendComment(QString comment);
     void sharePost();
+    void deletePost();
 
 protected:
     virtual void resizeEvent(QResizeEvent *event);
@@ -70,13 +76,16 @@ protected:
 private:
     QHBoxLayout *mainLayout;
     QVBoxLayout *leftColumnLayout;
+    QVBoxLayout *rightColumnLayout;
 
     QString postID;
+    QString postType;
+    bool postIsOwn;
 
     QLabel *postAuthorAvatar;
     QLabel *postAuthorName;
     QLabel *postCreatedAt;
-    QLabel *postType;
+    QLabel *postTypeLabel;
     QLabel *postRootAuthor;
     QLabel *postLikesCount;
     QLabel *postCommentsCount;
@@ -86,6 +95,7 @@ private:
     QPushButton *likeButton;
     QPushButton *commentButton;
     QPushButton *shareButton;
+    QPushButton *deleteButton;
 
     QTextBrowser *postText;
 
@@ -95,6 +105,7 @@ private:
 
 
     PumpController *pController;
+    Commenter *commenter;
 };
 
 #endif // POST_H
