@@ -41,7 +41,7 @@ public:
 
 
 signals:
-    void sharePost(QString postText); // TODO: params like Aspects/Public, pics...
+    void sharePost(QString postText); // TODO: params like Lists/Public, pics...
 
 public slots:
     void setMinimumMode();
