@@ -22,8 +22,9 @@
 
 PumpController::PumpController(QObject *parent) :  QObject(parent)
 {
-    this->userAgentString = "Dianara/0.4-dev";
+    this->userAgentString = "Dianara/0.5";
 
+    this->postsPerPage = 20;
 
     qoauth = new QOAuth::Interface();
     qoauth->setRequestTimeout(10000); // 10 sec timeout
@@ -68,10 +69,8 @@ PumpController::PumpController(QObject *parent) :  QObject(parent)
             this, SLOT(getInitialData()));
 
 
+
     qDebug() << "PumpController created";
-
-
-
 }
 
 
@@ -81,6 +80,15 @@ PumpController::~PumpController()
 }
 
 
+
+void PumpController::setPostsPerPage(int ppp)
+{
+    this->postsPerPage = ppp;
+    qDebug() << "PumpController: setting postsPerPage to" << this->postsPerPage;
+}
+
+
+
 /*
  * Set new user ID (user@domain.tld) and clear OAuth-related tokens/secrets
  *
@@ -133,6 +141,15 @@ void PumpController::setUserCredentials(QString userID)
 
 
 
+QString PumpController::currentUserID()
+{
+    return this->userID;
+}
+
+
+
+
+
 /*
  * Get any user's profile (not only our own)
  *
@@ -145,27 +162,10 @@ void PumpController::getUserProfile(QString userID)
 
     QString url = "https://" + splittedUserID.at(1) + "/api/user/" + splittedUserID.at(0);
 
-    QByteArray authorizationHeader = qoauth->createParametersString(url,
-                                                                 QOAuth::GET,
-                                                                 this->token,
-                                                                 this->tokenSecret,
-                                                                 QOAuth::HMAC_SHA1,
-                                                                 QOAuth::ParamMap(),
-                                                                 QOAuth::ParseForHeaderArguments);
-    qDebug() << authorizationHeader;
-
-    QNetworkRequest userProfileRequest;
-    userProfileRequest.setUrl(QUrl(url));
-    userProfileRequest.setRawHeader("Authorization", authorizationHeader);
-    userProfileRequest.setHeader(QNetworkRequest::ContentTypeHeader, "application/json");
-    userProfileRequest.setRawHeader("User-Agent",    userAgentString);
-
-    userProfileRequest.setAttribute(QNetworkRequest::User,
-                                    QVariant(UserProfileRequest));
-
+    QNetworkRequest userProfileRequest = this->prepareRequest(url, QOAuth::GET, UserProfileRequest);
     nam.get(userProfileRequest);
 
-    qDebug() << "Requested:" << userProfileRequest.url();
+    qDebug() << "Requested user profile:" << userProfileRequest.url();
 }
 
 
@@ -188,13 +188,10 @@ void PumpController::getAvatar(QString avatarURL)
 
 void PumpController::getImage(QString imageURL)
 {
-    qDebug() << "Getting image";
+    // FIXME: now we get BadRequest on public images on other hosts
+    // Probably need to use "proxyURL" image
 
-
-    QNetworkRequest imageRequest(QUrl((const QString)imageURL));
-    imageRequest.setRawHeader("User-Agent", userAgentString);
-    imageRequest.setAttribute(QNetworkRequest::User,
-                              QVariant(ImageRequest));
+    QNetworkRequest imageRequest = this->prepareRequest(imageURL, QOAuth::GET, ImageRequest);
 
     nam.get(imageRequest);
     qDebug() << "imageRequest sent";
@@ -212,26 +209,14 @@ void PumpController::getContactList()
     emit currentJobChanged(tr("Getting contact list..."));
     qDebug() << "Getting contact list";
 
-
     QString url = "https://" + this->serverURL + "/api/user/" +this->userName + "/following";
                                                                                 // also /followers?
 
-    QByteArray authorizationHeader = qoauth->createParametersString(url,
-                                                                 QOAuth::GET,
-                                                                 this->token,
-                                                                 this->tokenSecret,
-                                                                 QOAuth::HMAC_SHA1,
-                                                                 QOAuth::ParamMap(),
-                                                                 QOAuth::ParseForHeaderArguments);
+    QOAuth::ParamMap paramMap;
+    paramMap.insert("count", "100");  // TMP/FIXME get 100 contacts
 
-    QNetworkRequest contactListRequest;
-    contactListRequest.setUrl(QUrl(url));
-    contactListRequest.setRawHeader("Authorization", authorizationHeader);
-    contactListRequest.setHeader(QNetworkRequest::ContentTypeHeader, "application/json");
-    contactListRequest.setRawHeader("User-Agent",    userAgentString);
-
-    contactListRequest.setAttribute(QNetworkRequest::User,
-                                   QVariant(ContactListRequest));
+    QNetworkRequest contactListRequest = this->prepareRequest(url, QOAuth::GET,
+                                                              ContactListRequest, paramMap);
     nam.get(contactListRequest);
 }
 
@@ -239,33 +224,29 @@ void PumpController::getContactList()
 
 
 /*
- * GET https://pumpserver.example/api/username/inbox
+ * GET https://pumpserver.example/api/username/inbox/major
  *
  */
-void PumpController::getTimeline()
+void PumpController::getTimeline(int timelineOffset)
 {
     emit currentJobChanged(tr("Getting timeline..."));
-    qDebug() << "Getting timeline";
-
 
     QString url = "https://" + this->serverURL + "/api/user/" + this->userName + "/inbox/major";
 
-    QByteArray authorizationHeader = qoauth->createParametersString(url,
-                                                                 QOAuth::GET,
-                                                                 this->token,
-                                                                 this->tokenSecret,
-                                                                 QOAuth::HMAC_SHA1,
-                                                                 QOAuth::ParamMap(),
-                                                                 QOAuth::ParseForHeaderArguments);
+    QOAuth::ParamMap paramMap;
+    paramMap.insert("count", QString("%1").arg(this->postsPerPage).toLocal8Bit());
+    paramMap.insert("offset", QString("%1").arg(timelineOffset).toLocal8Bit());
 
-    QNetworkRequest timelineRequest;
-    timelineRequest.setUrl(QUrl(url));
-    timelineRequest.setRawHeader("Authorization", authorizationHeader);
-    timelineRequest.setHeader(QNetworkRequest::ContentTypeHeader, "application/json");
-    timelineRequest.setRawHeader("User-Agent",    userAgentString);
+    QNetworkRequest timelineRequest = this->prepareRequest(url, QOAuth::GET,
+                                                           TimelineRequest, paramMap);
 
-    timelineRequest.setAttribute(QNetworkRequest::User,
-                                 QVariant(TimelineRequest));
+
+    qDebug() << "===================================";
+    qDebug() << "Getting timeline" << url;
+    qDebug() << "with params:" << paramMap;
+    qDebug() << "\nAuthorization Header:" << timelineRequest.rawHeader("Authorization");
+    qDebug() << "\n\nFinal URL to retrieve:" << timelineRequest.url();
+    qDebug() << "===================================";
 
     nam.get(timelineRequest);
 }
@@ -294,11 +275,50 @@ void PumpController::getPost(int id)
 
 
 
+/*
+ * Prepare a QNetworkRequest with OAuth header, content type and user agent.
+ *
+ */
+QNetworkRequest PumpController::prepareRequest(QString url, QOAuth::HttpMethod method,
+                                               int requestType, QOAuth::ParamMap paramMap)
+{
+    QByteArray authorizationHeader = qoauth->createParametersString(url,
+                                                                 method,
+                                                                 this->token,
+                                                                 this->tokenSecret,
+                                                                 QOAuth::HMAC_SHA1,
+                                                                 paramMap,
+                                                                 QOAuth::ParseForHeaderArguments);
+
+    QNetworkRequest request;
+    // Don't append inline parameters if it's an image request, it messes up the filename
+    if (requestType != ImageRequest)
+    {
+        url.append(qoauth->inlineParameters(paramMap, QOAuth::ParseForInlineQuery));
+    }
+    request.setUrl(QUrl(url));
+    request.setRawHeader("Authorization", authorizationHeader);
+    request.setHeader(QNetworkRequest::ContentTypeHeader, "application/json");
+    request.setRawHeader("User-Agent", userAgentString);
+
+
+    request.setAttribute(QNetworkRequest::User,  QVariant(requestType));
+
+    return request;
+}
+
+
+
+/***************************************************************************/
+/***************************************************************************/
 /***************************************************************************/
 /***************************************************************************/
 /*********************************** SLOTS *********************************/
 /***************************************************************************/
 /***************************************************************************/
+/***************************************************************************/
+/***************************************************************************/
+
 
 
 
@@ -309,42 +329,41 @@ void PumpController::requestFinished(QNetworkReply *reply)
 
     qDebug() << "Request finished. HTTP code:" << httpCode;
     qDebug() << "Size:" << reply->size() << "bytes; URL:" << reply->url();
-
-    qDebug() << "isFinished()?" << reply->isFinished();
-    qDebug() << "Request type:" << requestType;
+    qDebug() << "isFinished()?" << reply->isFinished() << "; Request type:" << requestType;
 
     switch (httpCode)
     {
     case 503:
-        emit showNotification("Service Unavailable (503)");
+        emit showNotification("Service Unavailable (503)\n" + reply->url().toString());
         qDebug() << "HTTP 503: Service Unavailable.";
         qDebug() << "Data:  " << reply->readAll();
         return;
 
     case 500:
-        emit showNotification("Internal server error (500)");
+        emit showNotification("Internal server error (500)\n" + reply->url().toString());
         qDebug() << "HTTP 500: Internal server error.";
         qDebug() << "Data:  " << reply->readAll();
         return;
 
 
     case 404:
-        emit showNotification("Not Found (404)");
+        emit showNotification("Not Found (404)\n" + reply->url().toString());
         qDebug() << "HTTP 404: Not Found.";
         qDebug() << "Data:  " << reply->readAll();
         return;
 
     case 403:
-        emit showNotification("Forbidden (403)");
+        emit showNotification("Forbidden (403)\n" + reply->url().toString());
         qDebug() << "HTTP 403: Forbidden.";
         qDebug() << "Data:  " << reply->readAll();
         return;
 
 
     case 400:
-        emit showNotification("Bad Request (400)");
+        emit showNotification("Bad Request (400)\n" + reply->url().toString());
         qDebug() << "HTTP 400: Bad Request.";
         qDebug() << "Data:  " << reply->readAll();
+        //qDebug() << reply->attribute(QNetworkRequest::HttpReasonPhraseAttribute);
         return;
 
 
@@ -353,6 +372,48 @@ void PumpController::requestFinished(QNetworkReply *reply)
     }
 
 
+    if (httpCode == 200) // If OK received
+    {
+        switch (requestType) // Inform in status bar which operation was successful
+        {
+        case PublishPostRequest:
+            emit currentJobChanged(tr("Post published successfully."));
+            break;
+
+        case LikePostRequest:
+            emit currentJobChanged(tr("Post liked successfully."));
+            break;
+
+        case CommentPostRequest:
+            emit currentJobChanged(tr("Comment posted successfully."));
+            break;
+
+        case SharePostRequest:
+            emit currentJobChanged(tr("Post shared successfully."));
+            break;
+
+        case DeletePostRequest:
+            emit currentJobChanged(tr("Post deleted successfully."));
+            break;
+
+        case FollowContactRequest:
+            emit currentJobChanged(tr("Following successfully."));
+
+            // Re-request the contact list, now with a new contact (not very optimized...)
+            this->getContactList();
+            break;
+
+        case UnfollowContactRequest:
+            emit currentJobChanged(tr("Stopped following successfully."));
+
+            // Re-request the contact list, now with one contact less (not very optimized...)
+            this->getContactList();
+            break;
+
+        }
+    }
+
+
     // Read all received data
     QByteArray packetData = reply->readAll();
 
@@ -429,12 +490,15 @@ void PumpController::requestFinished(QNetworkReply *reply)
         if (jsonParsedOK && jsonData.size() > 0)
         {
             qDebug() << "JSON parsed OK";
-            emit currentJobChanged(tr("Timeline received."));
+            emit currentJobChanged(tr("Timeline received. Updating post list..."));
 
             jsonDataList = jsonData["items"].toList();
             qDebug() << "Number of items in inbox timeline:" << jsonDataList.size();
 
-            emit timeLineReceived(jsonDataList);
+            QString previousLink = jsonData["links"].toMap().value("prev").toMap().value("href").toString();
+            QString nextLink = jsonData["links"].toMap().value("next").toMap().value("href").toString();
+
+            emit timeLineReceived(jsonDataList, this->postsPerPage, previousLink, nextLink);
         }
         else
         {
@@ -649,7 +713,7 @@ void PumpController::authorizeApplication(QString verifierCode)
         this->isApplicationAuthorized = true;
 
         QSettings settings;
-        settings.setValue("isApplicationAuthorized", true);
+        settings.setValue("isApplicationAuthorized", this->isApplicationAuthorized);
         settings.setValue("token",       this->token);
         settings.setValue("tokenSecret", this->tokenSecret);
 
@@ -676,7 +740,7 @@ void PumpController::getInitialData()
 {
     qDebug() << "PumpController::getInitialData() step" << initialDataStep;
 
-    initialDataTimer->setInterval(5000);  // Every 5 sec
+    initialDataTimer->setInterval(2000);  // Every 2 sec
 
 
     switch (this->initialDataStep)
@@ -714,52 +778,227 @@ void PumpController::post(QString postText)
     qDebug() << "PumpController::post()";
 
 
+
     QString url = "https://" + this->serverURL + "/api/user/" + this->userName + "/feed";
 
-    QByteArray authorizationHeader = qoauth->createParametersString(url,
-                                                                 QOAuth::POST,
-                                                                 this->token,
-                                                                 this->tokenSecret,
-                                                                 QOAuth::HMAC_SHA1,
-                                                                 QOAuth::ParamMap(),
-                                                                 QOAuth::ParseForHeaderArguments);
-    qDebug() << authorizationHeader;
-
-    QNetworkRequest postRequest;
-    postRequest.setUrl(QUrl(url));
-    postRequest.setRawHeader("Authorization", authorizationHeader);
-    postRequest.setHeader(QNetworkRequest::ContentTypeHeader, "application/json");
-    postRequest.setRawHeader("User-Agent",    userAgentString);
+    QNetworkRequest postRequest = this->prepareRequest(url, QOAuth::POST, PublishPostRequest);
 
 
-    postRequest.setAttribute(QNetworkRequest::User,
-                             QVariant(PublishPostRequest));
+    QVariantMap jsonVariantObject;
+    jsonVariantObject.insert("objectType", "note");
+    jsonVariantObject.insert("content", postText);
+
+    QVariantMap jsonVariant;
+    jsonVariant.insert("verb", "post");
+    jsonVariant.insert("object", jsonVariantObject);
+
+    QJson::Serializer serializer;
+    bool ok;
+    QByteArray data = serializer.serialize(jsonVariant, &ok);
 
 
-    QByteArray data = QString("{"
-                              "\"verb\": \"post\","
-                              "\"object\": {"
-                              "              \"objectType\": \"note\","
-                              "              \"content\": \"%1\" "
-                              "            }"
-                              "}").arg(postText).toLocal8Bit();
-
-
-    qDebug() << "About to POST:" << postRequest.rawHeader("Authorization") << "\n" << data;
+    qDebug() << "About to POST:" << data;
 
     nam.post(postRequest, data);
 }
 
 
 
-
-void PumpController::likePost(QString postID)
+/*
+ * Like (favorite) a post, by its URL
+ *
+ */
+void PumpController::likePost(QString postID, QString postType)
 {
-    qDebug() << "Liking post" << postID;
+    qDebug() << "PumpController::likePost() liking post" << postID;
+
+    QString url =  "https://" + this->serverURL + "/api/user/" + this->userName + "/feed";
+
+    QNetworkRequest likeRequest = this->prepareRequest(url, QOAuth::POST, LikePostRequest);
+
+
+    QVariantMap jsonVariantObject;
+    jsonVariantObject.insert("objectType", postType);
+    jsonVariantObject.insert("id", postID);
+
+    QVariantMap jsonVariant;
+    jsonVariant.insert("verb", "favorite");
+    jsonVariant.insert("object", jsonVariantObject);
+
+    QJson::Serializer serializer;
+    bool ok;
+    QByteArray data = serializer.serialize(jsonVariant, &ok);
+
+    qDebug() << "about to POST:" << data;
+
+    nam.post(likeRequest, data);
 }
 
 
 
+void PumpController::addComment(QString comment, QString postID, QString postType)
+{
+    qDebug() << "PumpController::addComment() sending comment to this post:" << postID;
+
+    QString url =  "https://" + this->serverURL + "/api/user/" + this->userName + "/feed";
+
+    QNetworkRequest commentRequest = this->prepareRequest(url, QOAuth::POST, CommentPostRequest);
+
+
+
+    QVariantMap jsonVariantInReplyTo;
+    jsonVariantInReplyTo.insert("id", postID);
+    jsonVariantInReplyTo.insert("objectType", postType);
+
+    QVariantMap jsonVariantObject;
+    jsonVariantObject.insert("objectType", "comment");
+    jsonVariantObject.insert("content", comment);
+    jsonVariantObject.insert("inReplyTo", jsonVariantInReplyTo);
+
+    QVariantMap jsonVariant;
+    jsonVariant.insert("verb", "post");
+    jsonVariant.insert("object", jsonVariantObject);
+
+    QJson::Serializer serializer;
+    bool ok;
+    QByteArray data = serializer.serialize(jsonVariant, &ok);
+
+    qDebug() << "about to POST:" << data;
+
+    nam.post(commentRequest, data);
+}
+
+
+
+
+void PumpController::sharePost(QString postID, QString postType)
+{
+    qDebug() << "PumpController::sharePost() sharing post" << postID;
+
+    QString url =  "https://" + this->serverURL + "/api/user/" + this->userName + "/feed";
+
+    QNetworkRequest shareRequest = this->prepareRequest(url, QOAuth::POST, SharePostRequest);
+
+
+    QVariantMap jsonVariantObject;
+    jsonVariantObject.insert("objectType", postType);
+    jsonVariantObject.insert("id", postID);
+
+    QVariantMap jsonVariant;
+    jsonVariant.insert("verb", "share");
+    jsonVariant.insert("object", jsonVariantObject);
+
+    QJson::Serializer serializer;
+    bool ok;
+    QByteArray data = serializer.serialize(jsonVariant, &ok);
+
+    qDebug() << "about to POST:" << data;
+
+    nam.post(shareRequest, data);
+}
+
+
+
+void PumpController::deletePost(QString postID, QString postType)
+{
+    qDebug() << "PumpController::deletePost() deleting post" << postID;
+
+    QString url =  "https://" + this->serverURL + "/api/user/" + this->userName + "/feed";
+
+    QNetworkRequest deleteRequest = this->prepareRequest(url, QOAuth::POST, DeletePostRequest);
+
+
+    QVariantMap jsonVariantObject;
+    jsonVariantObject.insert("objectType", postType);
+    jsonVariantObject.insert("id", postID);
+
+    QVariantMap jsonVariant;
+    jsonVariant.insert("verb", "delete");
+    jsonVariant.insert("object", jsonVariantObject);
+
+    QJson::Serializer serializer;
+    bool ok;
+    QByteArray data = serializer.serialize(jsonVariant, &ok);
+
+    qDebug() << "about to POST:" << data;
+
+    nam.post(deleteRequest, data);
+}
+
+
+
+
+/*
+ * Add a contact to the /following list with their webfinger address
+ *
+ * This will trigger a re-request of the contact list, upon receiving the HTTP 200
+ * confirming the addition of the contact to /following
+ *
+ */
+void PumpController::followContact(QString address)
+{
+    qDebug() << "PumpController::followContact()" << address;
+
+    QString url =  "https://" + this->serverURL + "/api/user/" + this->userName + "/feed";
+
+    QNetworkRequest followRequest = this->prepareRequest(url, QOAuth::POST, FollowContactRequest);
+
+
+    QVariantMap jsonVariantObject;
+    jsonVariantObject.insert("objectType", "person");
+    jsonVariantObject.insert("id", "acct:" + address);
+
+    QVariantMap jsonVariant;
+    jsonVariant.insert("verb", "follow");
+    jsonVariant.insert("object", jsonVariantObject);
+
+    QJson::Serializer serializer;
+    bool ok;
+    QByteArray data = serializer.serialize(jsonVariant, &ok);
+
+    qDebug() << "about to POST:" << data;
+
+    nam.post(followRequest, data);
+}
+
+
+
+/*
+ * Remove a contact from the /following list with their webfinger address
+ *
+ * This will trigger a re-request of the contact list, upon receiving the HTTP 200
+ * confirming the removal of the contact from /following
+ *
+ */
+void PumpController::unfollowContact(QString address)
+{
+    qDebug() << "PumpController::unfollowContact()" << address;
+
+    QString url =  "https://" + this->serverURL + "/api/user/" + this->userName + "/feed";
+
+    QNetworkRequest unfollowRequest = this->prepareRequest(url, QOAuth::POST, UnfollowContactRequest);
+
+
+    QVariantMap jsonVariantObject;
+    jsonVariantObject.insert("objectType", "person");
+    jsonVariantObject.insert("id", "acct:" + address);
+
+    QVariantMap jsonVariant;
+    jsonVariant.insert("verb", "stop-following");
+    jsonVariant.insert("object", jsonVariantObject);
+
+    QJson::Serializer serializer;
+    bool ok;
+    QByteArray data = serializer.serialize(jsonVariant, &ok);
+
+    qDebug() << "about to POST:" << data;
+
+    nam.post(unfollowRequest, data);
+}
+
+
+
+
 void PumpController::setCurrentTimeline(QString link)
 {
     qDebug() << "PumpController::setCurrentTimeline()" << link;
