@@ -44,6 +44,7 @@
 #include <QVariant>
 #include <QVariantMap>
 #include <qjson/parser.h>
+#include <qjson/serializer.h>
 
 // For OAuth authentication
 #include <QtOAuth>
@@ -57,16 +58,24 @@ public:
     explicit PumpController(QObject *parent = 0);
     ~PumpController();
 
+    void setPostsPerPage(int ppp);
+
     void setNewUserID(QString userID);
     void setUserCredentials(QString userID);
+    QString currentUserID();
 
     void getUserProfile(QString userID);
     void getAvatar(QString avatarURL);
     void getImage(QString imageURL);
     void getContactList();
-    void getTimeline();
+    void getTimeline(int timelineOffset = 0);
     void getPost(int id);
 
+    QNetworkRequest prepareRequest(QString url,
+                                   QOAuth::HttpMethod method,
+                                   int requestType,
+                                   QOAuth::ParamMap paramMap = QOAuth::ParamMap());
+
     enum requestTypes
     {
         NoRequest,
@@ -81,7 +90,14 @@ public:
         AvatarRequest,
         ImageRequest,
 
-        PublishPostRequest
+        PublishPostRequest,
+        LikePostRequest,
+        CommentPostRequest,
+        SharePostRequest,
+        DeletePostRequest,
+
+        FollowContactRequest,
+        UnfollowContactRequest
     };
 
 
@@ -91,7 +107,8 @@ signals:
 
     void profileReceived(QString fullName, QString avatarURL);
     void contactListReceived(QVariantList contactList);
-    void timeLineReceived(QVariantList postList);
+    void timeLineReceived(QVariantList postList, int postsPerPage,
+                          QString previousLink, QString nextLink);
     void postReceived(QVariantMap post);
     void avatarPictureReceived(QByteArray pictureData, QUrl pictureURL);
     void imageReceived(QByteArray pictureData, QUrl pictureURL);
@@ -110,7 +127,13 @@ public slots:
     void getInitialData();
 
     void post(QString postText); // FIXME: more params
-    void likePost(QString postID);
+    void likePost(QString postID, QString postType);
+    void addComment(QString comment, QString postID, QString postType);
+    void sharePost(QString postID, QString postType);
+    void deletePost(QString postID, QString postType);
+
+    void followContact(QString address);
+    void unfollowContact(QString address);
 
     void setCurrentTimeline(QString link);
 
@@ -140,6 +163,8 @@ private:
     QTimer *initialDataTimer;
     int initialDataStep;
 
+
+    int postsPerPage;
 };
 
 #endif // PUMPCONTROLLER_H
