@@ -33,12 +33,12 @@ TimeLine::TimeLine(PumpController *pumpController, QWidget *parent) :  QWidget(p
     demoPostData.insert("text",   "<h2><b>Demo Post</b></h2><br>"
                                   "Press <b>F5</b> or <i>Update timelines</i> in the menu.<br><br>"
                                   "Upon startup, timeline should update automatically.<br><br>"
-                                  "<a href=\"http://dianara.nongnu.org\">Dianara's website</a><br><br>"
-                                  "<a href=\"http://jancoding.wordpress.com/dianara\">Dianara's blog</a>");
+                                  "<a href=\"http://jancoding.wordpress.com/dianara\">Dianara's blog</a><br>"
+                                  "<a href=\"http://dianara.nongnu.org\">Dianara's website</a><br><br>");
     demoPostData.insert("createdAt", "1970-01-01T01:02:03Z");
     demoPostData.insert("likesCount", "32");
-    demoPostData.insert("commentsCount", "256");
-    demoPostData.insert("resharesCount", "1024");
+    demoPostData.insert("commentsCount", "128");
+    demoPostData.insert("resharesCount", "256");
 
 
     splitter = new QSplitter(Qt::Vertical, this);
@@ -46,26 +46,44 @@ TimeLine::TimeLine(PumpController *pumpController, QWidget *parent) :  QWidget(p
     mainLayout->addWidget(splitter);
 
 
-    previousButton = new QPushButton(QIcon::fromTheme("go-previous"), tr("Previous Page"));
-    nextButton = new QPushButton(QIcon::fromTheme("go-next"), tr("Next Page"));
+    this->timelineOffset = 0;
+
+    firstPageButton = new QPushButton(QIcon::fromTheme("go-first"), tr("&First Page"));
+    connect(firstPageButton, SIGNAL(clicked()),
+            this, SLOT(goToFirstPage()));
+    previousPageButton = new QPushButton(QIcon::fromTheme("go-previous"), tr("&Previous Page"));
+    connect(previousPageButton, SIGNAL(clicked()),
+            this, SLOT(goToPreviousPage()));
+    nextPageButton = new QPushButton(QIcon::fromTheme("go-next"), tr("&Next Page"));
+    connect(nextPageButton, SIGNAL(clicked()),
+            this, SLOT(goToNextPage()));
 
     bottomLayout = new QHBoxLayout();
-    bottomLayout->addWidget(previousButton);
-    bottomLayout->addWidget(nextButton);
+    bottomLayout->addWidget(firstPageButton);
+    bottomLayout->addSpacing(16);
+    bottomLayout->addWidget(previousPageButton);
+    bottomLayout->addWidget(nextPageButton);
     mainLayout->addLayout(bottomLayout);
 
     // Add the default "demo" post
-    splitter->addWidget(new Post(pController, demoPostData, this));
+    splitter->addWidget(new Post(pController, demoPostData, QVariantList(), this));
 
 
     this->setLayout(mainLayout);
 
+
+    QSettings settings;
+    this->previousNewestPostID = settings.value("previousNewestPostID", "").toString();
+
     qDebug() << "TimeLine created";
 }
 
 
 TimeLine::~TimeLine()
 {
+    QSettings settings;
+    settings.setValue("previousNewestPostID", this->previousNewestPostID);
+
     qDebug() << "TimeLine destroyed";
 }
 
@@ -112,35 +130,46 @@ void TimeLine::clearTimeLineContents()
 /************************ SLOTS **************************/
 
 
-void TimeLine::setTimeLineContents(QVariantList postList)
+void TimeLine::setTimeLineContents(QVariantList postList, int postsPerPage,
+                                   QString previousLink, QString nextLink)
 {
     qDebug() << "TimeLine::setTimeLineContents()";
-    //qDebug() << postList;
-
 
     // Remove all previous posts in timeline
     qDebug() << "Removing previous posts from timeline";
     this->clearTimeLineContents();
 
 
+    this->postsPerPage = postsPerPage;
+
+    int newPostCount = 0;
+    bool allNewPostsCounted = false;
+    QString newestPostID; // Here we'll store the postID for the first (newest) post in the timeline
+                          // With it, we can know how many new posts (if any) we receive next time
+
     // Fill timeline with new contents
     foreach (QVariant singlePost, postList)
     {
         if (singlePost.type() == QVariant::Map)
         {
-            qDebug() << "Parsed a Map:";
+            this->previousPageLink = previousLink;   // Useless at the moment
+            this->nextPageLink = nextLink;
+
             QVariantMap singlePostMap = singlePost.toMap();
 
-            QString itemVerb = singlePostMap.value("verb").toString();  // if not "post", ignore ("favorite")
+            //QString itemVerb = singlePostMap.value("verb").toString();
 
+            QString postIsDeleted = singlePostMap.value("object").toMap().value("deleted").toString();
 
-            QVariantList postPhotosList;
 
             QString postID;
             QString postType;
             QString postAuthor;
             QString postAuthorAvatar;
             QString postAuthorId;
+            QString postAuthorHometown;
+            QString postAuthorBio;
+            QString postGenerator;
             QString postTitle;
             QString postImage;
             QString postText;
@@ -150,23 +179,50 @@ void TimeLine::setTimeLineContents(QVariantList postList)
             QString postLikesCount;
             QString postCommentsCount;
             QString postResharesCount;
-            QString postOembedThumbnailUrl;
+            QString postIsShared;
 
 
             postID = singlePostMap.value("object").toMap().value("id").toString();
+            if (newestPostID.isEmpty()) // only first time, for newest post
+            {
+                if (this->timelineOffset == 0)
+                {
+                    newestPostID = postID;
+                }
+                else
+                {
+                    newestPostID = this->previousNewestPostID;
+                    allNewPostsCounted = true;
+                }
+            }
+
+            if (!allNewPostsCounted)
+            {
+                if (postID == this->previousNewestPostID)
+                {
+                    allNewPostsCounted = true;
+                }
+                else
+                {
+                    ++newPostCount;
+                }
+            }
+
 
             postType = singlePostMap.value("object").toMap().value("objectType").toString();
-
-
-            // Only "note/image" types accepted, to avoid rendering "comment" posts as normal posts
-            // if (itemVerb == "post" && (postType == "note" || postType == "image"))
-            // Not needed anymore, since we now get /inbox/major
+            postIsShared = singlePostMap.value("object").toMap().value("pump_io").toMap().value("shared").toString();
+            //qDebug() << "post type:" << postType << "post is shared:" << postIsShared;
 
             postAuthor = singlePostMap.value("actor").toMap().value("displayName").toString();
             postAuthorAvatar = singlePostMap.value("actor").toMap().value("image").toMap().value("url").toString();
             // for Author ID, remove the first 5 characters from the field, "acct:"
             postAuthorId = singlePostMap.value("actor").toMap().value("id").toString().remove(0,5);
 
+            postAuthorBio = singlePostMap.value("actor").toMap().value("summary").toString();
+            postAuthorHometown = singlePostMap.value("actor").toMap().value("location").toMap().value("displayName").toString();
+
+            postGenerator = singlePostMap.value("generator").toMap().value("displayName").toString();
+
             postCreatedAt = singlePostMap.value("object").toMap().value("published").toString();
 
 
@@ -177,7 +233,15 @@ void TimeLine::setTimeLineContents(QVariantList postList)
             if (postType == "image")
             {
                 postTitle = singlePostMap.value("object").toMap().value("displayName").toString();
-                postImage = singlePostMap.value("object").toMap().value("image").toMap().value("url").toString();
+
+                // See if there's a proxy URL for the image first (for private images)
+                postImage = singlePostMap.value("object").toMap().value("image").toMap().value("pump_io").toMap().value("proxyURL").toString();
+
+                // If that's empty, use regular image->url field
+                if (postImage.isEmpty())
+                {
+                    postImage = singlePostMap.value("object").toMap().value("image").toMap().value("url").toString();
+                }
             }
 
 
@@ -187,44 +251,42 @@ void TimeLine::setTimeLineContents(QVariantList postList)
             postCommentsCount = singlePostMap.value("object").toMap().value("replies").toMap().value("totalItems").toString();
             postResharesCount = singlePostMap.value("object").toMap().value("shares").toMap().value("totalItems").toString();
 
+            // get comments list here, pass to the Post()
+            QVariantList commentsList = singlePostMap.value("object").toMap().value("replies").toMap().value("items").toList();
 
-            //qDebug() << "**** likes/comments/reshares:" << postLikesCount << postCommentsCount << postResharesCount;
-
-
-
-            // oEmbed stuff (youtube videos, etc.)
-            postOembedThumbnailUrl = singlePostMap.value("o_embed_cache").toMap().value("data").toMap().value("thumbnail_url").toString();
 
 
             QMap<QString,QString> postData;
             postData.insert("id",                 postID);
+            postData.insert("postType",           postType);
             postData.insert("author",             postAuthor);
             postData.insert("authorAvatar",       postAuthorAvatar);
             postData.insert("authorId",           postAuthorId);
+            postData.insert("authorBio",          postAuthorBio);
+            postData.insert("authorHometown",     postAuthorHometown);
+            postData.insert("generator",          postGenerator);
             postData.insert("title",              postTitle);
             postData.insert("image",              postImage);
             postData.insert("text",               postText);
             postData.insert("createdAt",          postCreatedAt);
-            postData.insert("postType",           postType);
             postData.insert("nsfw",               postIsNsfw);
             postData.insert("rootAuthor",         postRootAuthor);
             postData.insert("likesCount",         postLikesCount);
             postData.insert("commentsCount",      postCommentsCount);
             postData.insert("resharesCount",      postResharesCount);
-            postData.insert("oembedThumbnailUrl", postOembedThumbnailUrl);
+            postData.insert("postIsShared",       postIsShared);
 
 
-            // get comments list here, pass to the Post()  // WIP - FIXME
-            QVariantList commentsList = singlePostMap.value("last_three_comments").toList();
-            qDebug() << ">>>> Last 3 comments:\n";
-            foreach (QVariant comment, commentsList)
+
+            if (postIsDeleted.isEmpty())  // if post was NOT deleted
             {
-                qDebug() << comment.toMap().value("author").toMap().value("name");
-                qDebug() << comment.toMap().value("text");
-                qDebug() << "-------------------------------------";
+                this->splitter->addWidget(new Post(pController, postData, commentsList, this));
+            }
+            else
+            {
+                // if there's a "deleted" key, ignore this post
+                qDebug() << "This post was deleted on" << postIsDeleted << " / Not adding.";
             }
-
-            this->splitter->addWidget(new Post(pController, postData, this));
 
 
             /*
@@ -233,14 +295,59 @@ void TimeLine::setTimeLineContents(QVariantList postList)
             */
 
             // TMP FIXME, avoid unresizable posts (not enough height on splitter area)
-            //this->splitter->setMinimumHeight(this->splitter->height() + 200); // was 400
+            this->splitter->setMinimumHeight(this->splitter->height() + 200); // was 400
         }
         else  // singlePost.type() is not a QVariant::Map
         {
             qDebug() << "Expected a Map, got something else";
             qDebug() << postList;
         }
-
     }
+
+    emit scrollToTop(); // ask mainWindow to scroll the QScrollArea containing the timeline to the top
+
+    this->previousNewestPostID = newestPostID;
+
+    qDebug() << "New posts:" << newPostCount << "; Newest post ID:" << previousNewestPostID;
+
+    emit timelineRendered(newPostCount);
+
     qDebug() << "setTimeLineContents() /END";
 }
+
+
+
+
+
+void TimeLine::goToFirstPage()
+{
+    qDebug() << "TimeLine::goToFirstPage()";
+    this->pController->getTimeline();
+
+    this->timelineOffset = 0; // Keep timeline offset coherent
+}
+
+
+
+void TimeLine::goToPreviousPage()
+{
+    qDebug() << "TimeLine::goToPreviousPage()";
+
+    this->timelineOffset -= this->postsPerPage;
+    if (timelineOffset < 0)
+    {
+        timelineOffset = 0;
+    }
+    this->pController->getTimeline(this->timelineOffset);
+}
+
+
+
+void TimeLine::goToNextPage()
+{
+    qDebug() << "TimeLine::goToNextPage()";
+
+    this->timelineOffset += this->postsPerPage;
+
+    this->pController->getTimeline(this->timelineOffset);
+}
