@@ -66,7 +66,7 @@ QString Timestamp::fuzzyTime(QString isoTime)
                          - dateTimeConverter.toTime_t();
     //qDebug() << "time difference in seconds:" << timeDifference;
 
-    QString localTimeDateString = tr("About %1 ago");
+    QString localTimeDateString = QString("About %1 ago");
 
     // At this point, timeDifference is in SECONDS
 
